from typing import Callable

import numpy as np

import pyewjn.noise.zeta
import pyewjn.util


def get_im_ref_p(eps: Callable[[float], complex]) -> Callable[[float], float]:
	zeta_p = pyewjn.noise.zeta.get_zeta_p_function(eps)

	def im_ref_p(u: float) -> float:
		zeta_p_val = zeta_p(u)
		return np.imag((np.pi * 1j * u - zeta_p_val) / (np.pi * 1j * u + zeta_p_val))

	return im_ref_p


def get_im_ref_s(eps_t: Callable[[float], complex]) -> Callable[[float], float]:
	def integrand(kappa: float, u: float) -> complex:
		k_eff2 = kappa**2 + u**2
		k_eff4 = k_eff2**2
		k_eff = k_eff2 ** (1 / 2)
		return eps_t(k_eff) / k_eff4

	def im_ref_s(u: float) -> float:
		integral = pyewjn.util.complex_quad(
			lambda kappa: integrand(kappa, u), 0, np.inf, epsabs=1e-12
		)
		return (1 / (4 * u**2)) * ((4 * u**3 * integral) - 1)

	return im_ref_s
